@@ -4,6 +4,7 @@ Entity baseclass, metaclass and descriptor
 
 from sqlalchemy                     import Table, Integer, desc, deferred
 from sqlalchemy.ext.assignmapper    import assign_mapper
+from sqlalchemy.util                import OrderedDict
 from elixir.statements              import Statement
 from elixir.fields                  import Field
 from elixir.options                 import options_defaults
@@ -23,7 +24,7 @@ __pudge_all__ = __all__
 
 DEFAULT_AUTO_PRIMARYKEY_NAME = "id"
 DEFAULT_AUTO_PRIMARYKEY_TYPE = Integer
-
+DEFAULT_VERSION_ID_COL = "row_version"
 
 class EntityDescriptor(object):
     '''
@@ -52,7 +53,7 @@ class EntityDescriptor(object):
                 else:
                     self.parent = base
 
-        self.fields = dict()
+        self.fields = OrderedDict()
         self.relationships = dict()
         self.constraints = list()
 
@@ -63,12 +64,13 @@ class EntityDescriptor(object):
 
         # set default value for options
         self.order_by = None
-        self.tablename = None
         self.table_args = list()
         self.metadata = getattr(self.module, 'metadata', elixir.metadata)
 
         for option in ('inheritance', 
-                       'autoload', 'shortnames', 'auto_primarykey'):
+                       'autoload', 'tablename', 'shortnames', 
+                       'auto_primarykey',
+                       'version_id_col'):
             setattr(self, option, options_defaults[option])
 
         for option_dict in ('mapper_options', 'table_options'):
@@ -90,6 +92,8 @@ class EntityDescriptor(object):
                 modulename = entity.__module__.replace('.', '_')
                 tablename = "%s_%s" % (modulename, entity.__name__)
                 self.tablename = tablename.lower()
+        elif callable(self.tablename):
+            self.tablename = self.tablename(entity)
     
     def setup(self):
         '''
@@ -139,6 +143,9 @@ class EntityDescriptor(object):
         if self.order_by:
             kwargs['order_by'] = self.translate_order_by(self.order_by)
         
+        if self.version_id_col:
+            kwargs['version_id_col'] = self.fields[self.version_id_col].column
+
         if self.parent:
             if self.inheritance == 'single':
                 # at this point, we don't know whether the parent relationships
@@ -160,7 +167,6 @@ class EntityDescriptor(object):
         assign_mapper(session.context, self.entity, self.entity.table,
                       properties=properties, **kwargs)
 
-
     def setup_table(self):
         '''
         Create a SQLAlchemy table-object with all columns that have been 
@@ -192,6 +198,11 @@ class EntityDescriptor(object):
 #                for field in self.fields.itervalues():
 #                    self.add_field(field)
 
+        if self.version_id_col:
+            if not isinstance(self.version_id_col, basestring):
+                self.version_id_col = DEFAULT_VERSION_ID_COL
+            self.add_field(Field(Integer, colname=self.version_id_col))
+
         if not self.autoload:
             if not self.primary_keys and self.auto_primarykey:
                 self.create_auto_primary_key()
@@ -257,8 +268,7 @@ class EntityDescriptor(object):
                             "relation in entity '%s'. You should specify "
                             "inverse relations manually by using the inverse "
                             "keyword."
-                            % (rel.name, rel.entity.__name__) 
-                          )
+                            % (rel.name, rel.entity.__name__))
         # When a matching inverse is found, we check that it has only
         # one relation matching as its own inverse. We don't need the result
         # of the method though. But we do need to be careful not to start an
@@ -268,7 +278,6 @@ class EntityDescriptor(object):
 
         return matching_rel
 
-    @property
     def all_relationships(self):
         if self.parent:
             res = self.parent._descriptor.all_relationships
@@ -276,6 +285,7 @@ class EntityDescriptor(object):
             res = dict()
         res.update(self.relationships)
         return res
+    all_relationships = property(all_relationships)
 
     def setup_relationships(cls):
         for relationship in list(EntityDescriptor.uninitialized_rels):
@@ -283,6 +293,28 @@ class EntityDescriptor(object):
                 EntityDescriptor.uninitialized_rels.remove(relationship)
     setup_relationships = classmethod(setup_relationships)
 
+class EntityMeta(type):
+    """
+    Entity meta class.
+    """
+
+    def __init__(cls, name, bases, dict_):
+        # only process subclasses of Entity, not Entity itself
+        if bases[0] is object:
+            return
+
+        # create the entity descriptor
+        desc = cls._descriptor = EntityDescriptor(cls)
+        EntityDescriptor.current = desc
+        
+        # process statements
+        Statement.process(cls)
+        
+        # setup misc options here (like tablename etc.)
+        desc.setup_options()
+        
+        # create table & assign (empty) mapper
+        desc.setup()
 
 class Entity(object):
     '''
@@ -306,23 +338,6 @@ class Entity(object):
     For further information, please refer to the provided examples or
     tutorial.
     '''
-    
-    class __metaclass__(type):
-        def __init__(cls, name, bases, dict_):
-            # only process subclasses of Entity, not Entity itself
-            if bases[0] is object:
-                return
 
-            # create the entity descriptor
-            desc = cls._descriptor = EntityDescriptor(cls)
-            EntityDescriptor.current = desc
-            
-            # process statements
-            Statement.process(cls)
-            
-            # setup misc options here (like tablename etc.)
-            desc.setup_options()
-            
-            # create table & assign (empty) mapper
-            desc.setup()
+    __metaclass__ = EntityMeta
 
