@@ -39,13 +39,16 @@ The list of supported arguments are as follows:
 | ``metadata``        | Specify a custom MetaData                             |
 +---------------------+-------------------------------------------------------+
 | ``autoload``        | Automatically load column definitions from the        |
-|                     | existing database table.  For now, it does **not**    |
-|                     | work for ``has_and_belongs_to_many`` relationships.   |
+|                     | existing database table.                              |
 |                     | Using autoloaded tables implies setting               |
 |                     | ``delay_setup`` to ``True`` before defining your      |
 |                     | entities.                                             |
 +---------------------+-------------------------------------------------------+
-| ``tablename``       | Specify a custom tablename                            |
+| ``tablename``       | Specify a custom tablename. You can either provide a  |
+|                     | plain string or a callable. The callable will be      |
+|                     | given the entity (ie class) as argument and must      |
+|                     | return a string representing the name of the table    |
+|                     | for that entity.                                      |
 +---------------------+-------------------------------------------------------+
 | ``shortnames``      | Usually tablenames include the full module-path       |
 |                     | to the entity, but lower-cased and separated by       |
@@ -62,6 +65,13 @@ The list of supported arguments are as follows:
 |                     | corresponding entity.  If this option is False,       |
 |                     | it will disallow auto-creation of a primary key.      |
 +---------------------+-------------------------------------------------------+
+| ``version_id_col``  | If this option is True, it will create a version      |
+|                     | column automatically using the default name. If given |
+|                     | as string, it will create the column using that name. |
+|                     | This can be used to prevent concurrent modifications  |
+|                     | to the entity's table rows (i.e. it will raise an     |
+|                     | exception if it happens).                             |
++---------------------+-------------------------------------------------------+
 | ``order_by``        | How to order select results. Either a string or a     |
 |                     | list of strings, composed of the field name,          |
 |                     | optionally lead by a minus (descending order).        |
@@ -110,7 +120,9 @@ options_defaults = dict(
     inheritance='single',
     autoload=None,
     shortnames=False,
+    tablename=None,
     auto_primarykey=True,
+    version_id_col=False,
     mapper_options=dict(),
     table_options=dict(),
 )
@@ -123,6 +135,7 @@ class UsingOptions(object):
         'tablename',
         'shortnames',
         'auto_primarykey',
+        'version_id_col',
         'order_by',
     )
     
@@ -132,15 +145,20 @@ class UsingOptions(object):
         for kwarg in kwargs:
             if kwarg in UsingOptions.valid_options:
                 setattr(desc, kwarg, kwargs[kwarg])
+            else:
+                raise Exception("'%s' is not a valid option for Elixir "
+                                "entities." % kwarg)
 
 
-class UsingTableOptions(object):    
+class UsingTableOptions(object):
+
     def __init__(self, entity, *args, **kwargs):
         entity._descriptor.table_args = list(args)
         entity._descriptor.table_options.update(kwargs)
 
 
 class UsingMapperOptions(object):
+
     def __init__(self, entity, *args, **kwargs):
         entity._descriptor.mapper_options.update(kwargs)
 
